#include "IOExpanderCentralWidget.h"
#include "ErrorHandling.h"

#include <QBitArray>
#include <QList>

CIOExpanderCentralWidget::CIOExpanderCentralWidget( CIOExpanderCommunicationModule & commModule )
:m_commModule(commModule)
{
	SetupLayout();
	SetupSignals();
}

CIOExpanderCentralWidget::~CIOExpanderCentralWidget()
{

}

void CIOExpanderCentralWidget::setPage(int index)
{
	m_contentsWidget->setCurrentRow( index );
}

void CIOExpanderCentralWidget::showPaneel(int show)
{
	if(show>=0)
		m_contentsWidget->setVisible(show);
	else
	{
		bool v = m_contentsWidget->isVisible();
		m_contentsWidget->setVisible(v^1);
	}
}

void CIOExpanderCentralWidget::SetupLayout()
{
//instantiate here new tabs

	m_ioControlTab = new CIOWidget;
	m_patternTab = new CPatternWidget;
	m_spiTab = new CSPIWidget;
	m_servoTab = new CServoWidget;

	m_contentsWidget = new QListWidget();
	m_contentsWidget->setViewMode(QListView::IconMode);
	m_contentsWidget->setIconSize(QSize(96, 84));
	m_contentsWidget->setMovement(QListView::Static);
	m_contentsWidget->setFixedWidth(128);
	m_contentsWidget->setSpacing(12);

	CreateIcons();


	m_pagesWidget = new QStackedWidget;


	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		CIOExpanderWidget * widget = CIOExpanderWidget::s_widgets[i];
		m_pagesWidget->addWidget(widget);
	}

	QHBoxLayout	*	contentLayout = new QHBoxLayout;
	QVBoxLayout	*	layout = new QVBoxLayout(this);

	contentLayout->addWidget( m_contentsWidget );
	contentLayout->addWidget( m_pagesWidget );


	layout->addLayout(contentLayout);

	m_contentsWidget->setCurrentRow(0);

	setLayout(layout);
}

void CIOExpanderCentralWidget::SetupSignals()
{
	// needed for changing pages
	connect( m_contentsWidget, SIGNAL( currentItemChanged(QListWidgetItem *, QListWidgetItem *) ), this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)) );

	//commands
	connect( m_patternTab, SIGNAL(RequestSamples(quint8,quint16,quint16,quint16)), &m_commModule, SLOT(RequestSamples(quint8, quint16,quint16,quint16)));
	connect( m_patternTab, SIGNAL(SendPattern(QBitArray*, quint16,quint16,quint16)), &m_commModule, SLOT(SendPattern(QBitArray *, quint16,quint16,quint16)));
	connect( m_patternTab, SIGNAL(RequestSamplesAndSendPattern(QBitArray *, quint16,quint16,quint16)), &m_commModule, SLOT(RequestSamplesAndSendPattern(QBitArray *, quint16,quint16,quint16)));

	connect( m_ioControlTab, SIGNAL(SetDigitalOutput(quint16)), &m_commModule, SLOT(SetOutput(quint16)));
	connect( m_ioControlTab, SIGNAL(SetRelais(quint8)), &m_commModule, SLOT(SetRelais(quint8)));
	connect( m_ioControlTab, SIGNAL(PollDigitalInput()), &m_commModule, SLOT(PollInput()));
	connect( m_ioControlTab, SIGNAL(PollADC(quint8)), &m_commModule, SLOT(PollADC(quint8)));

	connect( m_spiTab, SIGNAL(SendSPIData(quint8, quint8, QList<quint8>)), &m_commModule, SLOT(SendSPIData(quint8, quint8, QList<quint8> )));

	connect( m_servoTab, SIGNAL(setServos(QList<float>,float,float)),&m_commModule,SLOT(SetServos(QList<float>,float,float)));
	connect(m_servoTab,SIGNAL(setOutput(quint16)),&m_commModule,SLOT(SetOutput(quint16)));

	connect( &m_commModule, SIGNAL(NewSamplesReceived(QBitArray*)), m_patternTab, SLOT(UpdateInputSamples(QBitArray*)) );
	connect( &m_commModule, SIGNAL(NewInputReceived(quint16)), m_ioControlTab, SLOT(NewDigitalInputReceived(quint16)) );
	connect( &m_commModule, SIGNAL(NewADCValueReceived(quint8, quint8)), m_ioControlTab, SLOT(NewADCValueReceived(quint8,quint8)) );
	connect( &m_commModule, SIGNAL(NewSPIDataReceived(quint8)), m_spiTab, SLOT(SpiDataReceived(quint8)));
}

void	CIOExpanderCentralWidget::CreateIcons()
{
	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		CIOExpanderWidget * widget = CIOExpanderWidget::s_widgets[i];
		QListWidgetItem *item = new QListWidgetItem(m_contentsWidget);
		item->setIcon(QIcon(tr("Data\\") + widget->iconName() ));
		item->setText( widget->title() );
		item->setTextAlignment(Qt::AlignHCenter);
		item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	}
}


void CIOExpanderCentralWidget::changePage(QListWidgetItem *current, QListWidgetItem *previous)
{
	if (!current)
		current = previous;

	int currentIndex = m_pagesWidget->currentIndex();

	if(currentIndex < CIOWidget::s_widgets.size())
		CIOWidget::s_widgets[ currentIndex ]->HideWidgetCallback();

	m_pagesWidget->setCurrentIndex( m_contentsWidget->row(current) );

	currentIndex = m_contentsWidget->row(current);

	if(currentIndex < CIOWidget::s_widgets.size())
		CIOWidget::s_widgets[ currentIndex ]->ShowWidgetCallback();

	emit pageSelected( m_contentsWidget->row(current) );
}
