#include "IOExpanderMainWindow.h"
#include "ErrorHandling.h"


CIOExpanderMainWindow::CIOExpanderMainWindow()
{
	connectionLost();

	m_commModule = new CIOExpanderCommunicationModule;
	setupLayout();
	setupMenu();
	setupShortCuts();
	setupSignals();

}

void CIOExpanderMainWindow::setupLayout()
{
	m_centralWidget = new CIOExpanderCentralWidget( *m_commModule );
	m_statusBar = new CIOExpanderStatusBar;

	setCentralWidget( m_centralWidget );
	setStatusBar( m_statusBar );

	m_commModule->start();
}

void CIOExpanderMainWindow::setupSignals()
{
	connect( m_commModule, SIGNAL(ConnectionLost()), m_statusBar, SLOT(ConnectionLost()) );
	connect( m_commModule, SIGNAL(ConnectionEstablished()), m_statusBar, SLOT(ConnectionEstablished()) );
	connect( m_commModule, SIGNAL(ConnectionLost()), this, SLOT(connectionLost()) );
	connect( m_commModule, SIGNAL(NewSWVersion(QString)), this, SLOT(saveSoftwareVersion(QString)) );


	connect( CErrorHandling::GetErrorObject(), SIGNAL(ErrorProcessed(const QString&,int)), m_statusBar, SLOT(showMessage(const QString&, int)) );

	connect( m_viewActionGroup, SIGNAL(triggered ( QAction * ) ), this, SLOT(viewMenuChanged ( QAction *  )));
	connect( m_showPaneelAction, SIGNAL(toggled(bool)),this,SLOT(showPaneel(bool)));

	connect( m_centralWidget, SIGNAL( pageSelected(int)), this, SLOT(updateViewCheckboxes(int)));

	connect( m_togglePaneelShortCut, SIGNAL(activated()), m_showPaneelAction, SLOT(toggle()));

	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		connect( m_quickSelectShortCuts[i], SIGNAL(activated()), m_viewActions[i], SLOT(trigger()));
		if(i == 8)
			break;
	}
}

void CIOExpanderMainWindow::setupMenu()
{
	m_fileMenu = menuBar()->addMenu(tr("&Datei"));

	m_fileMenu->addAction(tr("&Reset"),m_commModule,SLOT(Reset()));
	m_fileMenu->addSeparator();
	m_fileMenu->addAction(tr("&Beenden"),this,SLOT(close()));


	m_viewMenu = menuBar()->addMenu(tr("&Ansicht"));
	m_viewActionGroup = new QActionGroup(this);

	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		QAction *action = new QAction( CIOExpanderWidget::s_widgets[i]->title(),m_viewActionGroup );
		action->setCheckable(1);
		if(i==0)
			action->setChecked(1);

		m_viewMenu->addAction(action);
		m_viewActions.append(action);
	}

	m_viewMenu->addSeparator();

	m_showPaneelAction = new QAction(tr("&Paneel"),this);
	m_showPaneelAction->setCheckable(1);
	m_showPaneelAction->setChecked(1);
	m_viewMenu->addAction(m_showPaneelAction);

	m_helpMenu = menuBar()->addMenu(tr("&Hilfe"));
	m_helpMenu->addAction(tr("&Software Version"),this,SLOT(showSoftwareVersion()));
	m_helpMenu->addAction(tr("&Tastenkombinationen"),this,SLOT(showShortcutsMessage()));

	m_helpMenu->addSeparator();
	m_helpMenu->addAction(tr("&ber"),this,SLOT(showAboutMessage()));
}

void CIOExpanderMainWindow::setupShortCuts()
{
	m_togglePaneelShortCut = new QShortcut(QKeySequence(tr("Ctrl+P")),this);

	#define min(a,b) a<b?a:b

	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		QShortcut	*sc = new QShortcut(QKeySequence(tr("Ctrl+") + QString::number(i+1)),this);
		m_quickSelectShortCuts << sc;

		if(i == 8)
			break;
	}
}

void CIOExpanderMainWindow::viewMenuChanged ( QAction * action )
{
	for(int i = 0; i < m_viewActions.size();i++)
	{
		if( m_viewActions[i] == action)
		{
			m_centralWidget->setPage(i);
			break;
		}
	}
}

void CIOExpanderMainWindow::showPaneel(bool show)
{
	m_centralWidget->showPaneel(show);
}

void CIOExpanderMainWindow::updateViewCheckboxes(int n)
{
	if( m_viewActions.size() <= n)
		return;

	m_viewActions[n]->setChecked(1);
}

void CIOExpanderMainWindow::showAboutMessage()
{
	QString e_m_a_i_l = ("miwitt001");
	e_m_a_i_l += ("@");
	e_m_a_i_l += ("gm");
	e_m_a_i_l += ("x.de");
	QString message =
	tr("<b>USB IO Expander Control</b><br><br>Michael Wittmann<br>2008/2009<br><hr><br>") +
	tr("Diese Software darf einschlielich der zugehrigen Hardware fr private Zwecke (bei gewerblicher Nutzung bitte per Email anfragen) frei und kostenlos verwendet werden. ") +
	tr("Ich gebe jedoch keinerlei Garantie auf die Funktionsfhigkeit bzw. Fehlerfreiheit dieses Produktes.<br><hr><br>") +
	tr("Bei Fragen, Anregungen, Lob und Kritik freue ich mich ber eine Email an:\n<a href=\"mailto:") + e_m_a_i_l +
	tr("\">") + e_m_a_i_l + tr("</a>");

	QMessageBox box(QMessageBox::Information,tr("About"), message, QMessageBox::Close);
	box.exec();
}

void CIOExpanderMainWindow::showShortcutsMessage()
{
	QString message = tr("<b>Tastenkombinationen</b><hr><br>");
	QString space = tr("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");

	for(int i = 0; i < CIOExpanderWidget::s_widgets.size();i++)
	{
		message += tr("Strg+") + QString::number(i+1) + space + CIOExpanderWidget::s_widgets[i]->title() + tr("<br>");

		if(i == 8)
			break;
	}
	message += tr("<br>Strg+P") + space + ("Paneel ein-/ausblenden");

	QMessageBox box(QMessageBox::Information,tr("Tastenkombinationen"), message, QMessageBox::Close);
	box.exec();
}

void CIOExpanderMainWindow::showSoftwareVersion()
{
	QString message = tr("IOExpanderControl: <b>2.00</b><hr>") + tr("Firmware: <b>") + m_softwareVer +tr("</b>");

	QMessageBox box(QMessageBox::Information,tr("Software Version"), message, QMessageBox::Close);
	box.exec();
}

void CIOExpanderMainWindow::saveSoftwareVersion(QString sw)
{
	m_softwareVer = sw;
}

void CIOExpanderMainWindow::connectionLost()
{
	m_softwareVer = tr("N/A");
}

void CIOExpanderMainWindow::closeEvent(QCloseEvent *event)
{
	for(int i = 0; i < CIOWidget::s_widgets.size();i++)
	{
		CIOWidget::s_widgets[i]->CloseWidgetCallback();
	}

	event->accept();
}

