#include "IOWidget.h"
#include "../ErrorHandling.h"

CIOWidget::CIOWidget()
:CIOExpanderWidget(tr("I/O"),tr("IO.png"))
{
	m_autoSetOutputState = 0;
	m_autoPollInputState = 0;
	m_autoSetRelaisState = 0;
	m_autoPollADCState = 0;

	m_inputTimerFrequ = 1000;
	m_ADCTimerFrequ = 1000;

	m_pollInputTimer = new QTimer(this);
	m_pollADCTimer = new QTimer(this);

	SetupLayout();
	SetupSignals();
}

CIOWidget::~CIOWidget()
{

}

void CIOWidget::ShowWidgetCallback()
{
	if(m_autoSetOutputState)
	{
		quint16 o = GetOutputFromGui();
		emit SetDigitalOutput(o);
	}

	if(m_autoSetRelaisState)
	{
		quint8 r = GetRelaisStateFromGui();
		emit SetRelais(r);
	}

	if(m_autoPollInputState)
	{
		emit PollDigitalInput();
		m_pollInputTimer->start();
	}

	if(m_autoPollADCState)
	{
		m_pollADCTimer->start();

		quint8 ch = 0;

		ch |= m_ADCChannelCheckbox[0]->isChecked();
		ch |= m_ADCChannelCheckbox[1]->isChecked() << 1;
		ch |= m_ADCChannelCheckbox[2]->isChecked() << 2;
		ch |= m_ADCChannelCheckbox[3]->isChecked() << 3;

		if(ch)
			emit PollADC(ch);
	}
}

void CIOWidget::HideWidgetCallback()
{
	m_pollInputTimer->stop();
	m_pollADCTimer->stop();
}

void CIOWidget::CloseWidgetCallback()
{
	m_pollInputTimer->stop();
	m_pollADCTimer->stop();

	emit SetDigitalOutput(0);
	emit SetRelais(0);
}

void	CIOWidget::SetupLayout()
{
	//Layouts
	QGridLayout	*	layout = new QGridLayout(this);
	//QVBoxLayout	*	outLayout = new QVBoxLayout;
	QGridLayout	*	outLayout = new QGridLayout;
	//QVBoxLayout	*	inLayout = new QVBoxLayout;
	QGridLayout	*	inLayout = new QGridLayout;
	QVBoxLayout	*	relaisLayout = new QVBoxLayout;
	QVBoxLayout	*	adcLayout = new QVBoxLayout;
	QVBoxLayout	*	outCtrlLayout = new QVBoxLayout;
	QVBoxLayout	*	inCtrlLayout = new QVBoxLayout;
	QVBoxLayout	*	relaisCtrlLayout = new QVBoxLayout;
	QVBoxLayout	*	adcCtrlLayout = new QVBoxLayout;

	//Group Boxes
	QGroupBox	*	outGroupBox = new QGroupBox(tr("Output"));
	QGroupBox	*	inGroupBox = new QGroupBox(tr("Input"));
	QGroupBox	*	relaisGroupBox = new QGroupBox(tr("Relais"));
	QGroupBox	*	adcGroupBox = new QGroupBox(tr("ADC"));

	QGroupBox	*	outCtrlGroupBox = new QGroupBox(tr("Output Steuerung"));
	QGroupBox	*	inCtrlGroupBox = new QGroupBox(tr("Input Steuerung"));
	QGroupBox	*	relaisCtrlGroupBox = new QGroupBox(tr("Relais Steuerung"));
	QGroupBox	*	adcCtrlGroupBox = new QGroupBox(tr("ADC Steuerung"));

	//Button Groups
	m_relaisGroup = new QButtonGroup;
	m_relaisGroup->setExclusive(0);
	m_inGroup = new QButtonGroup;
	m_inGroup->setExclusive(0);
	m_outGroup = new QButtonGroup;
	m_outGroup->setExclusive(0);

	//Init in/out checkboxes
	for(int n = 0; n < 16; n++)
	{
		m_outBoxes[n] = new QCheckBox(tr("Out ") + QString::number(n));
		//outLayout->addWidget(m_outBoxes[n]);
		outLayout->addWidget(m_outBoxes[n], n%8, n/8);
		m_outGroup->addButton(m_outBoxes[n],n);

		m_inBoxes[n] = new QCheckBox(tr("In ") + QString::number(n));
		m_inBoxes[n]->setDisabled(1);
		//inLayout->addWidget(m_inBoxes[n]);
		inLayout->addWidget(m_inBoxes[n], n%8, n/8);
		m_inGroup->addButton(m_inBoxes[n],n);
	}

	//init relais checkboxes
	for(int k = 0; k < 4;k++)
	{
		m_relaisBoxes[k] = new QCheckBox( tr("Relais ") + QString::number(k+1));
		relaisLayout->addWidget(m_relaisBoxes[k]);
		m_relaisGroup->addButton(m_relaisBoxes[k],k);
	}
	relaisLayout->addStretch(10);

	//init adc lcds
	for(int j = 0; j < 4; j++)
	{
		m_ADCChannelCheckbox[j] = new QCheckBox(tr("Kanal ") + QString::number(j+1));
		adcLayout->addWidget( m_ADCChannelCheckbox[j] );
		m_ADCLcds[j] = new QLCDNumber();
		adcLayout->addWidget( m_ADCLcds[j] );
	}
	adcLayout->addStretch(10);

	//init buttons, sliders etc...
	m_setOutputButton = new QPushButton(tr("Ausgnge setzen"));
 	m_autoSetOutputCheckbox = new QCheckBox(tr("automatisch setzen"));

	m_pollInputButton = new QPushButton(tr("Eingnge pollen"));
	m_autoPollInputCheckbox = new QCheckBox(tr("automatisch pollen"));
	m_pollInputTimerSlider = new QSlider(Qt::Horizontal);
	m_pollInputTimerSlider->setRange(1,50);
	m_pollInputTimerSlider->setSliderPosition(10);
	m_pollInputTimerLabel = new QLabel(tr("Intervall: 1000 ms"));

	m_setRelaisButton = new QPushButton(tr("Relais schalten"));
	m_autoSetRelaisCheckbox = new QCheckBox(tr("sofort schalten"));

	m_pollADCButton = new QPushButton(tr("ADC pollen"));
	m_autoPollADCCheckbox = new QCheckBox(tr("automatisch pollen"));
	m_pollADCTimerSlider = new QSlider(Qt::Horizontal);
	m_pollADCTimerSlider->setRange(1,50);
	m_pollADCTimerSlider->setSliderPosition(10);
	m_pollADCTimerLabel = new QLabel(tr("Intervall: 1000 ms"));

	//setup rest of the output layout
	outCtrlLayout->addWidget(m_setOutputButton);
	outCtrlLayout->addWidget(m_autoSetOutputCheckbox);
	outCtrlLayout->addStretch(10);

	//setup rest of the input layout
	inCtrlLayout->addWidget(m_pollInputButton);
	inCtrlLayout->addWidget(m_autoPollInputCheckbox);
	inCtrlLayout->addWidget(m_pollInputTimerLabel);
	inCtrlLayout->addWidget(m_pollInputTimerSlider);
	inCtrlLayout->addStretch(10);

	//setup rest of the relais layout
	relaisCtrlLayout->addWidget(m_setRelaisButton);
	relaisCtrlLayout->addWidget(m_autoSetRelaisCheckbox);
	relaisCtrlLayout->addStretch(10);

	//setup rest of the Adc layout
	adcCtrlLayout->addWidget(m_pollADCButton);
	adcCtrlLayout->addWidget(m_autoPollADCCheckbox);
	adcCtrlLayout->addWidget(m_pollADCTimerLabel);
	adcCtrlLayout->addWidget(m_pollADCTimerSlider);
	adcCtrlLayout->addStretch(10);

	//setup group boxes
	outGroupBox->setLayout(outLayout);
	inGroupBox->setLayout(inLayout);
	relaisGroupBox->setLayout(relaisLayout);
	adcGroupBox->setLayout(adcLayout);

	outCtrlGroupBox->setLayout(outCtrlLayout);
	inCtrlGroupBox->setLayout(inCtrlLayout);
	relaisCtrlGroupBox->setLayout(relaisCtrlLayout);
	adcCtrlGroupBox->setLayout(adcCtrlLayout);

	//setup main layout
	layout->addWidget(outGroupBox,0,0);
	layout->addWidget(inGroupBox,0,1);
	layout->addWidget(relaisGroupBox,0,2);
	layout->addWidget(adcGroupBox,0,3);

	layout->setRowStretch(1,10);

	layout->addWidget(outCtrlGroupBox,2,0);
	layout->addWidget(inCtrlGroupBox,2,1);
	layout->addWidget(relaisCtrlGroupBox,2,2);
	layout->addWidget(adcCtrlGroupBox,2,3);

	layout->setColumnStretch(0,1);
	layout->setColumnStretch(1,1);
	layout->setColumnStretch(2,1);
	layout->setColumnStretch(3,1);

	setLayout(layout);

	m_autoSetOutputCheckbox->setChecked(1);
	SetOutputsAutoCheckbox(1);
	m_autoSetRelaisCheckbox->setChecked(1);
	SetRelaisAutoCheckbox(1);
}

void	CIOWidget::SetupSignals()
{
	//input output relais buttons
	connect(m_setOutputButton,SIGNAL(clicked()),this,SLOT(SetOutputsButton()));
	connect(m_pollInputButton,SIGNAL(clicked()),this,SLOT(PollInputsButton()));
	connect(m_setRelaisButton,SIGNAL(clicked()),this,SLOT(SetRelaisButton()));
	connect(m_pollADCButton,SIGNAL(clicked()),this,SLOT(PollADCButton()));

	//Group signal for auto updating
	connect(m_outGroup,SIGNAL(buttonClicked(int)),this,SLOT(OutputsChanged(int)));
	connect(m_relaisGroup,SIGNAL(buttonClicked(int)),this,SLOT(RelaisChanged(int)));

	//checkboxes
	connect(m_autoPollInputCheckbox,SIGNAL(clicked(bool)),this,SLOT(PollInputsAutoCheckbox(bool)));
	connect(m_autoSetOutputCheckbox,SIGNAL(clicked(bool)),this,SLOT(SetOutputsAutoCheckbox(bool)));
	connect(m_autoSetRelaisCheckbox,SIGNAL(clicked(bool)),this,SLOT(SetRelaisAutoCheckbox(bool)));
	connect(m_autoPollADCCheckbox,SIGNAL(clicked(bool)),this,SLOT(PollADCAutoCheckbox(bool)));

	//slider
	connect(m_pollInputTimerSlider,SIGNAL(valueChanged(int)),this,SLOT(InputTimerSlider(int)));
	connect(m_pollADCTimerSlider,SIGNAL(valueChanged(int)),this,SLOT(ADCTimerSlider(int)));

	//timer
	connect(m_pollInputTimer,SIGNAL(timeout()),this,SLOT(PollInputsButton()));
	connect(m_pollADCTimer,SIGNAL(timeout()),this,SLOT(PollADCButton()));
}

void CIOWidget::SetOutputsButton()
{
	quint16 o = GetOutputFromGui();

	emit SetDigitalOutput(o);
}

void CIOWidget::SetOutputsAutoCheckbox(bool b)
{
	if( b )
	{
		SetOutputsButton();
		m_setOutputButton->setDisabled(1);
		m_autoSetOutputState = 1;
	}
	else
	{
		m_setOutputButton->setDisabled(0);
		m_autoSetOutputState = 0;
	}
}

void CIOWidget::OutputsChanged(int n)
{
	if(m_autoSetOutputState)
		SetOutputsButton();
}

void CIOWidget::PollInputsButton()
{
	emit PollDigitalInput();
}

void CIOWidget::PollInputsAutoCheckbox(bool b)
{
	if( b )
	{
		m_pollInputButton->setDisabled(1);
		m_autoPollInputState = 1;
		m_pollInputTimer->start(m_inputTimerFrequ);
	}
	else
	{
		m_pollInputButton->setDisabled(0);
		m_autoPollInputState = 0;
		m_pollInputTimer->stop();
	}
}

void CIOWidget::InputTimerSlider(int value)
{
	m_inputTimerFrequ = value * 100;
	m_pollInputTimerLabel->setText( tr("Intervall: ") + QString::number(m_inputTimerFrequ) + tr(" ms"));

	if(m_pollInputTimer->isActive())
		m_pollInputTimer->setInterval(m_inputTimerFrequ);
}

void CIOWidget::PollADCButton()
{
	quint8 ch = 0;

	ch |= m_ADCChannelCheckbox[0]->isChecked();
	ch |= m_ADCChannelCheckbox[1]->isChecked() << 1;
	ch |= m_ADCChannelCheckbox[2]->isChecked() << 2;
	ch |= m_ADCChannelCheckbox[3]->isChecked() << 3;

	if(ch)
		emit PollADC(ch);
	else
		SHOW_INFO(tr("Kein ADC-Kanal ausgewhlt"));
}

void CIOWidget::PollADCAutoCheckbox(bool b)
{
	if( b )
	{
		m_pollADCButton->setDisabled(1);
		m_autoPollADCState = 1;
		m_pollADCTimer->start(m_ADCTimerFrequ);
	}
	else
	{
		m_pollADCButton->setDisabled(0);
		m_autoPollADCState = 0;
		m_pollADCTimer->stop();
	}
}

void CIOWidget::ADCTimerSlider(int value)
{
	m_ADCTimerFrequ = value * 100;
	m_pollADCTimerLabel->setText( tr("Intervall: ") + QString::number(m_ADCTimerFrequ) + tr(" ms"));

	if(m_pollADCTimer->isActive())
		m_pollADCTimer->setInterval(m_ADCTimerFrequ);
}

void CIOWidget::SetRelaisButton()
{
	quint8 r = GetRelaisStateFromGui();
	emit SetRelais(r);
}

void CIOWidget::SetRelaisAutoCheckbox(bool b)
{
	if( b )
	{
		SetRelaisButton();
		m_setRelaisButton->setDisabled(1);
		m_autoSetRelaisState = 1;
	}
	else
	{
		m_setRelaisButton->setDisabled(0);
		m_autoSetRelaisState = 0;
	}
}

void CIOWidget::RelaisChanged(int n)
{
	if(m_autoSetRelaisState)
	{
		SetRelaisButton();
	}
}

void	CIOWidget::NewDigitalInputReceived(quint16 state)
{
	for(int n = 0; n < 16;n++)
	{
		m_inBoxes[n]->setChecked(state & (1<<n));
	}
}

void	CIOWidget::NewADCValueReceived(quint8 value, quint8 channel)
{
	if(channel < 4)
		m_ADCLcds[channel]->display(value);
}

quint16 CIOWidget::GetOutputFromGui()
{
	quint16 r = 0;
	for(int n = 0; n < 16;n++)
		r |= ((m_outBoxes[n]->isChecked() ? 1 : 0) << n);
	return r;
}

quint8	CIOWidget::GetRelaisStateFromGui()
{
	quint16 r = 0;
	for(int n = 0; n < 4;n++)
		r |= ((m_relaisBoxes[n]->isChecked() ? 1 : 0) << n);

	return r;
}
