#pragma once
#include <QtGui>
#include <QtGlobal>
#include <QString>

#include "../IOExpanderWidget.h"

class CIOWidget
: public CIOExpanderWidget
{
Q_OBJECT

public:
	CIOWidget();
	~CIOWidget();

	void ShowWidgetCallback();
	void HideWidgetCallback();

public slots:
	void	NewDigitalInputReceived(quint16 state);
	void	NewADCValueReceived(quint8 value, quint8 channel);
	void	CloseWidgetCallback();

signals:
	void 	SetDigitalOutput(quint16 out);
	void 	SetRelais(quint8 out);
	void 	PollDigitalInput();
	void 	PollADC(quint8);

private:
	void	SetupLayout();
	void	SetupSignals();

	quint16 GetOutputFromGui();
	quint8	GetRelaisStateFromGui();

private slots:
	void SetOutputsButton();
	void SetOutputsAutoCheckbox(bool b);
	void OutputsChanged(int n);

	void PollInputsButton();
	void PollInputsAutoCheckbox(bool b);
	void InputTimerSlider(int value);

	void SetRelaisButton();
	void SetRelaisAutoCheckbox(bool b);
	void RelaisChanged(int n);

	void PollADCButton();
	void PollADCAutoCheckbox(bool b);
	void ADCTimerSlider(int value);

private:
	QCheckBox	*	m_inBoxes[16];
	QCheckBox	*	m_outBoxes[16];
	QCheckBox	*	m_relaisBoxes[4];

	QLCDNumber	*	m_ADCLcds[4];
	QCheckBox	*	m_ADCChannelCheckbox[4];

	QButtonGroup	*m_outGroup;
	QButtonGroup	*m_inGroup;
	QButtonGroup	*m_relaisGroup;
	QButtonGroup	*m_ADCGroup;

	QPushButton	*	m_setOutputButton;
	QCheckBox	*	m_autoSetOutputCheckbox;

	QPushButton	*	m_pollInputButton;
	QCheckBox	*	m_autoPollInputCheckbox;
	QSlider		*	m_pollInputTimerSlider;
	QLabel		*	m_pollInputTimerLabel;

	QPushButton	*	m_setRelaisButton;
	QCheckBox	*	m_autoSetRelaisCheckbox;

	QPushButton	*	m_pollADCButton;
	QCheckBox	*	m_autoPollADCCheckbox;
	QSlider		*	m_pollADCTimerSlider;
	QLabel		*	m_pollADCTimerLabel;

private:
	int				m_inputTimerFrequ;
	int				m_ADCTimerFrequ;

	bool			m_autoSetOutputState;
	bool			m_autoPollInputState;
	bool			m_autoSetRelaisState;
	bool			m_autoPollADCState;

	QTimer	*		m_pollInputTimer;
	QTimer	*		m_pollADCTimer;
};
