#include "PatternWidget.h"
#include "../ErrorHandling.h"
#include <QList>

#define		NUMBER_OF_SAMPLES		100



CPatternViewWidget::CPatternViewWidget(QWidget * parent)
:QGraphicsView(parent)
{
    setRenderHint(QPainter::Antialiasing);

	m_scene = new QGraphicsScene;

	int h = (CSinglePattern::s_dataPointHeight + CSinglePattern::s_borderSize) * 32 ;
    m_scene->setSceneRect(0, 0, width(), h);

	setAutoFillBackground(1);
	setCacheMode(QGraphicsView::CacheBackground);
	setViewportUpdateMode(FullViewportUpdate);

	setScene(m_scene);
}

CPatternViewWidget::~CPatternViewWidget()
{

}

void CPatternViewWidget::drawBackground(QPainter *painter, const QRectF &rect)
{
	float h = CSinglePattern::s_dataPointHeight + CSinglePattern::s_borderSize;

	for(int n = 0; n < 32;)
	{
		painter->setPen(Qt::darkGray);
		painter->setBrush(Qt::darkGray);
		painter->drawRect(QRectF(rect.left(),h*n,rect.width(),h));
		n++;
		painter->setPen(Qt::lightGray);
		painter->setBrush(Qt::lightGray);
		painter->drawRect(QRectF(rect.left(),h*n,rect.width(),h));
		n++;
	}

	if(rect.top() < 0)
	{
		painter->setPen(Qt::lightGray);
		painter->setBrush(Qt::lightGray);
		painter->drawRect(QRectF(rect.left(),rect.top(),rect.width(), -rect.top()));
	}
	if(rect.bottom() > 32*h)
	{
		painter->setPen(Qt::darkGray);
		painter->setBrush(Qt::darkGray);
		painter->drawRect(QRectF(rect.left(),32*h,rect.width(), rect.bottom()));
	}
}

void	CPatternViewWidget::removePatterns(EPatternType type)
{
	QList<QGraphicsItem *> items = m_scene->items();

	foreach(QGraphicsItem *item,items)
	{
		if(type == dynamic_cast<CSinglePattern*>(item)->Type() )
			m_scene->removeItem(item);
		else if(type == PATTERN_UNDEFINED)
			m_scene->removeItem(item);
	}
}

void	CPatternViewWidget::SetInputPattern(QBitArray p[16])
{
	removePatterns(PATTERN_IN);
	int sceneWidth = CSinglePattern::s_dataPointWidth * p[0].size() > width() ? CSinglePattern::s_dataPointWidth * p[0].size() : width();
	m_scene->setSceneRect(0, 0, sceneWidth , (CSinglePattern::s_dataPointHeight + CSinglePattern::s_borderSize) * 32 );

	for( int n = 0; n < 16;n++)
	{
		CSinglePattern* pat = new CSinglePattern(p[n], n, PATTERN_IN);
		pat->setPos(0, CSinglePattern::s_patternHeight * n);
		m_scene->addItem(pat);
	}
	update();
}

void	CPatternViewWidget::SetOutputPattern(QBitArray p[16])
{
	removePatterns(PATTERN_OUT);
	int sceneWidth = CSinglePattern::s_dataPointWidth * p[0].size() > width() ? CSinglePattern::s_dataPointWidth * p[0].size() : width();
	m_scene->setSceneRect(0, 0, sceneWidth, (CSinglePattern::s_dataPointHeight + CSinglePattern::s_borderSize) * 32 );

	for( int n = 0; n < 16;n++)
	{
		CSinglePattern* pat = new CSinglePattern(p[n], n,PATTERN_OUT);
		pat->setPos(0, CSinglePattern::s_patternHeight * (n+16));
		m_scene->addItem(pat);
	}
	update();
}
