#include "PatternWidget.h"
#include "../ErrorHandling.h"

CPatternWidget::CPatternWidget()
:CIOExpanderWidget(tr("Pattern"),tr("Pattern.png"))
{
	SetupLayout();
	SetupSignals();
}

CPatternWidget::~CPatternWidget()
{

}

void CPatternWidget::ShowWidgetCallback()
{

}

void CPatternWidget::HideWidgetCallback()
{

}

void CPatternWidget::CloseWidgetCallback()
{

}

void CPatternWidget::SetupLayout()
{
	//labels
	QLabel		*	m_numSamplesLabel = new QLabel(tr("Anzahl der Samples"));
	QLabel		*	frequLabel = new QLabel(tr("Frequenz"));

	//groupbox
	QGroupBox	*	ctrlGroup = new QGroupBox(tr("Pattern Steuerung"));

	//pattern widget
	m_pattern = new CPatternViewWidget;

	//buttons
	m_defineOutputPatternButton = new QPushButton(tr("Output definieren"));
	m_defineTriggerButton = new QPushButton(tr("Trigger definieren"));
	m_requestButton = new QPushButton(tr("Samplen"));
	m_saveButton = new QPushButton(tr("Speichern"));
	m_loadButton = new QPushButton(tr("Laden"));

	//spinbox
	m_numSamplesSpinBox = new QSpinBox();
	m_numSamplesSpinBox->setRange(1,200);
	m_numSamplesSpinBox->setValue(1);
	m_frequencySpinBox = new QSpinBox();
	m_frequencySpinBox->setRange(1,100000);

	//checkboxes
	m_useInputCheckBox = new QCheckBox(tr("Input"));
	m_useOutputCheckBox = new QCheckBox(tr("Output"));

	//Layouts
	QHBoxLayout	*	layout = new QHBoxLayout(this);
	QVBoxLayout	*	ctrlLayout = new QVBoxLayout;

	//fill button layout
	ctrlLayout->addWidget(m_defineOutputPatternButton);
	ctrlLayout->addWidget(m_defineTriggerButton);
	ctrlLayout->addStretch(10);
	ctrlLayout->addWidget(frequLabel);
	ctrlLayout->addWidget(m_frequencySpinBox);
	ctrlLayout->addWidget(m_numSamplesLabel);
	ctrlLayout->addWidget(m_numSamplesSpinBox);
	ctrlLayout->addWidget(m_useOutputCheckBox);
	ctrlLayout->addWidget(m_useInputCheckBox);
	ctrlLayout->addWidget(m_requestButton);
	ctrlLayout->addWidget(m_saveButton);
	ctrlLayout->addWidget(m_loadButton);

	//setup group boxes
	ctrlGroup->setLayout(ctrlLayout);

	layout->addWidget(m_pattern);
	layout->addWidget(ctrlGroup);

	setLayout(layout);

	m_defDialog = new CPatternDefinitionWindow(this);
	m_trigDefDialog = new CTriggerDefinitionWindow(this);
}

void CPatternWidget::SetupSignals()
{
	connect(m_requestButton,SIGNAL(clicked()),this,SLOT(RequestButton()));
	connect(m_saveButton,SIGNAL(clicked()),this,SLOT(SavePatternToDisk()));
	connect(m_loadButton,SIGNAL(clicked()),this,SLOT(LoadPatternFromDisk()));
	connect(m_defineOutputPatternButton,SIGNAL(clicked()),this,SLOT(DefineOutputPatternButton()));
	connect(m_defineTriggerButton,SIGNAL(clicked()),this,SLOT(DefineTriggerButton()));
	connect(m_useOutputCheckBox,SIGNAL(clicked(bool)),this,SLOT(SelectOutput(bool)));
}

void CPatternWidget::SavePatternToDisk()
{
	QString filename = QFileDialog:: getSaveFileName(this,QString(),QString(),tr("Pattern Files(*.pat)"));
	QFile	file (filename);

	if (!file.open(QIODevice::WriteOnly))
	{
		SHOW_INFO(tr("Konnte Datei \"") + filename + tr("\" nicht ffnen") );
		return;
	}

	QDataStream out(&file);

	for(int i = 0; i < 16; i++)
	{
		out << m_inputPattern[i];
	}
	for(int i = 0; i < 16; i++)
	{
		out << m_outputPattern[i];
	}

	out << m_frequencySpinBox->value();
	out << m_trigger1;
	out << m_trigger2;
	out << m_numSamplesSpinBox->value();

	out << (int)m_useOutputCheckBox->isChecked();
	out << (int)m_useInputCheckBox->isChecked();
}

void CPatternWidget::LoadPatternFromDisk()
{
	QString filename = QFileDialog:: getOpenFileName(this,QString(),QString(),tr("Pattern Files(*.pat)"));
	QFile	file (filename);

	if (!file.open(QIODevice::ReadOnly))
	{
		SHOW_INFO(tr("Konnte Datei \"") + filename + tr("\" nicht ffnen") );
		return;
	}

	QDataStream in(&file);

	for(int i = 0; i < 16; i++)
	{
		in >> m_inputPattern[i];
	}
	m_pattern->SetInputPattern(m_inputPattern);

	for(int i = 0; i < 16; i++)
	{
		in >> m_outputPattern[i];
	}
	m_pattern->SetOutputPattern(m_outputPattern);

	int frequ;
	in >> frequ;
	m_frequencySpinBox->setValue(frequ);

	in >> m_trigger1;
	in >> m_trigger2;

	m_trigDefDialog->SetTrigger(m_trigger1,m_trigger2);

	int samples;
	in >> samples;
	m_numSamplesSpinBox->setValue(samples);

	int useOut;
	in >> useOut;
	m_useOutputCheckBox->setChecked(useOut);

	int useIn;
	in >> useIn;

	m_useInputCheckBox->setChecked(useIn);
}

void CPatternWidget::UpdateInputSamples(QBitArray *pattern)
{
	for(int n = 0;n < 16;n++)
		m_inputPattern[n] = pattern[n];

	m_pattern->SetInputPattern(pattern);
}

void CPatternWidget::RequestButton()
{
	if(m_useInputCheckBox->isChecked() && m_useOutputCheckBox->isChecked())
	{
		//clear input pattern
		for(int n = 0;n < 16;n++)
			m_inputPattern[n].clear();
		m_pattern->SetInputPattern(m_inputPattern);

		if(m_outputPattern->size() == 0)
			SHOW_INFO(tr("Kein Output-Pattern definiert"));
		else
			emit RequestSamplesAndSendPattern(m_outputPattern, m_frequencySpinBox->value(), m_trigger1, m_trigger2);
	}
	else if( m_useInputCheckBox->isChecked())
	{
		//clear input pattern
		for(int n = 0;n < 16;n++)
			m_inputPattern[n].clear();
		m_pattern->SetInputPattern(m_inputPattern);
		emit RequestSamples( m_numSamplesSpinBox->value(), m_frequencySpinBox->value(), m_trigger1, m_trigger2 );
	}
	else if(m_useOutputCheckBox->isChecked())
	{
		//clear input pattern
		for(int n = 0;n < 16;n++)
			m_inputPattern[n].clear();
		m_pattern->SetInputPattern(m_inputPattern);

		if(m_outputPattern->size() == 0)
			SHOW_INFO(tr("Kein Output-Pattern definiert"));
		else
			emit SendPattern(m_outputPattern, m_frequencySpinBox->value(), m_trigger1, m_trigger2 );
	}
	else
		SHOW_INFO(tr("Kein Input/Output gewhlt"));
}

void CPatternWidget::SelectOutput(bool b)
{
	m_numSamplesSpinBox->setDisabled(b);
}

void CPatternWidget::DefineOutputPatternButton()
{
	//if( m_defDialog->exec() == QDialog::Accepted)
	m_defDialog->exec();
	{
		QBitArray *p = m_defDialog->GetPattern();

		for(int n = 0; n < 16; n++)
		{
			if(p)
				m_outputPattern[n] = p[n];
			else
				m_outputPattern[n].clear();
		}
		m_pattern->SetOutputPattern(m_outputPattern);
	}
}

void CPatternWidget::DefineTriggerButton()
{
//	if( m_trigDefDialog->exec() == QDialog::Accepted)
	m_trigDefDialog->exec();
	{
		m_trigDefDialog->GetTrigger(m_trigger1,m_trigger2);
	}
}

