#include "SinglePattern.h"


CSinglePattern::CSinglePattern()
{
	m_type = PATTERN_UNDEFINED;
	m_patternShape = 0;
	m_id = 0;
	m_pen.setColor(Qt::red);

	QBitArray	a;
	a.fill(0,100);
	for(int i = 0; i < 100; i+=2)
		a.setBit(i,1);

	BuildPattern(a);
}

CSinglePattern::CSinglePattern(QBitArray p, int id, EPatternType type)
{
	m_patternShape = 0;
	m_id = id;
	m_type = type;;

	if(m_type == PATTERN_IN)
		m_pen.setColor(Qt::red);
	else if(m_type == PATTERN_OUT)
		m_pen.setColor(Qt::green);
	else
		m_pen.setColor(Qt::white);

	BuildPattern(p);
}

CSinglePattern::~CSinglePattern()
{
	if(m_patternShape)
		delete m_patternShape;
}

QRectF CSinglePattern::boundingRect () const
{
	return QRectF(0,0,m_pattern.size() * s_dataPointWidth,s_dataPointHeight + s_borderSize );
}

void CSinglePattern::BuildPattern(QBitArray	p)
{
	if(m_patternShape)
		delete m_patternShape;

	m_patternShape = new QPainterPath;

	m_pattern = p;
	float y = s_borderSize/2.0f;
	bool t = 0;

	for( int i = 0; i < m_pattern.size();i++)
	{
		float x = s_dataPointWidth * (i+1);

		if(m_pattern.at(i))
		{
			if(t)
				m_patternShape->lineTo(x, y);
			else
				m_patternShape->moveTo(x, y);

			m_patternShape->lineTo(x + s_dataPointWidth, y);
		}
		else
		{
			if(t)
				m_patternShape->lineTo(x, y + s_dataPointHeight);
			else
				m_patternShape->moveTo(x, y + s_dataPointHeight);

			m_patternShape->lineTo(x + s_dataPointWidth, y + s_dataPointHeight);
		}
		if( (i != (m_pattern.size() - 1)) && (m_pattern.at(i) != m_pattern.at(i+1)) )
			t = 1;
		else
			t = 0;
	}
}

bool	CSinglePattern::Type()
{
	return m_type;
}

void CSinglePattern::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
	painter->setPen(m_pen);

	if(m_patternShape)
		painter->drawPath(*m_patternShape);

	if( !m_pattern.isEmpty())
	{
		painter->setPen(Qt::black);
		QRectF textRect(0,0,s_dataPointWidth,s_dataPointHeight+s_borderSize);
		painter->drawText(textRect,Qt::AlignVCenter | Qt::AlignHCenter,QString::number(m_id));
	}
}


const int		CSinglePattern::s_dataPointHeight = 10;
const int		CSinglePattern::s_dataPointWidth = 13;
const int		CSinglePattern::s_borderSize = 6;
const int 		CSinglePattern::s_patternHeight = s_dataPointHeight + s_borderSize;
