#include "TriggerDefinitionWindow.h"


CTriggerDefinitionWindow::CTriggerDefinitionWindow(QWidget * parent)
:QDialog(parent,Qt::WindowTitleHint | Qt::WindowSystemMenuHint)
{
	SetupLayout();SetupSignals();
}

CTriggerDefinitionWindow::~CTriggerDefinitionWindow()
{

}

void CTriggerDefinitionWindow::SetupLayout()
{
	m_okCancelButtons = new QDialogButtonBox( QDialogButtonBox::Ok /*| QDialogButtonBox::Cancel*/ );

	QGridLayout	*	layout = new QGridLayout;

	for(int n = 0; n < 16; n++)
	{
		m_boxes[n] = new QComboBox();
		m_boxes[n]->addItem(tr("Kein Trigger"));
		m_boxes[n]->addItem(tr("Steigende Flanke"));
		m_boxes[n]->addItem(tr("Fallende Flanke"));
		m_boxes[n]->addItem(tr("Jede Flanke"));
		QLabel * label = new QLabel(tr("Bit") + QString::number(n));
		layout->addWidget(label,n,0);
		layout->addWidget(m_boxes[n],n,1);
	}
	layout->addWidget(m_okCancelButtons,16,0,1,2);
	setLayout(layout);
}

void CTriggerDefinitionWindow::SetupSignals()
{
	connect(m_okCancelButtons, SIGNAL(accepted()), this, SLOT(accept()));
    connect(m_okCancelButtons, SIGNAL(rejected()), this, SLOT(reject()));
}

void CTriggerDefinitionWindow::GetTrigger(quint16 &t1, quint16 &t2)
{
	for(int n = 0; n < 16; n++)
	{
		quint16	mask = 1<<n;
		switch( m_boxes[n]->currentIndex() )
		{
			case 0: //trigger off
			{
			}break;
			case 1: //rising edge
			{
				t1 |= mask;
			}break;
			case 2: //falling edge
			{
				t2 |= mask;
			}break;
			case 3: //every edge
			{
				t1 |= mask;
				t2 |= mask;
			}break;
			default:
			{
				t1 = t2 = 0;
				return;
			}break;
		};
	}
}

void CTriggerDefinitionWindow::SetTrigger(quint16 t1, quint16 t2)
{
	for(int n = 0; n < 16; n++)
	{
		quint16	mask = 1<<n;

		if((t1 & mask) && (t2 & mask))
		{
			m_boxes[n]->setCurrentIndex(3);
		}
		else if(t1 & mask)
		{
			m_boxes[n]->setCurrentIndex(1);
		}
		else if(t2 & mask)
		{
			m_boxes[n]->setCurrentIndex(2);
		}
		else
		{
			m_boxes[n]->setCurrentIndex(0);
		}

		switch( m_boxes[n]->currentIndex() )
		{
			case 0: //trigger off
			{
			}break;
			case 1: //rising edge
			{
				t1 |= mask;
			}break;
			case 2: //falling edge
			{
				t2 |= mask;
			}break;
			case 3: //every edge
			{
				t1 |= mask;
				t2 |= mask;
			}break;
			default:
			{
				t1 = t2 = 0;
				return;
			}break;
		};
	}
}
