#include "SPIWidget.h"
#include "../ErrorHandling.h"


CSPIWidget::CSPIWidget()
:CIOExpanderWidget(tr("SPI"),tr("SPI.png"))
{
	m_dataValidator = new QRegExpValidator(this);
	SetupLayout();
	SetupSignals();
}

CSPIWidget::~CSPIWidget()
{
}

void CSPIWidget::ShowWidgetCallback()
{

}

void CSPIWidget::HideWidgetCallback()
{

}

void CSPIWidget::CloseWidgetCallback()
{

}

void	CSPIWidget::SetupLayout()
{
	QGroupBox		*	dataOrderBox = new QGroupBox("Bit-Reihenfolge");
	QHBoxLayout		*	dataOrderLayout = new QHBoxLayout;
	QRadioButton	*	lsbFirstButton = new QRadioButton("LSB zuerst");
	QRadioButton	*	msbFirstButton = new QRadioButton("MSB zuerst");
	m_dataOrderConfigGroup = new QButtonGroup;

	dataOrderLayout->addWidget(lsbFirstButton);
	m_dataOrderConfigGroup->addButton( lsbFirstButton, 0 );
	dataOrderLayout->addWidget(msbFirstButton);
	m_dataOrderConfigGroup->addButton( msbFirstButton, 1 );
	dataOrderBox->setLayout(dataOrderLayout);
	lsbFirstButton->setChecked(1);


	QGroupBox		*	clockPolBox = new QGroupBox("Clock-Polaritt",this);
	QHBoxLayout		*	clockPolLayout = new QHBoxLayout;
	QRadioButton	*	clockPolRisingButton = new QRadioButton("LOW im Ruhezustand, fhrende Flanke steigend");
	QRadioButton	*	clockPolFallingButton = new QRadioButton("HIGH im Ruhezustand, fhrende Flanke fallend");
	m_clockPolConfigGroup = new QButtonGroup;

	clockPolLayout->addWidget(clockPolRisingButton);
	m_clockPolConfigGroup->addButton( clockPolRisingButton, 0 );
	clockPolLayout->addWidget(clockPolFallingButton);
	m_clockPolConfigGroup->addButton( clockPolFallingButton, 1 );
	clockPolBox->setLayout(clockPolLayout);
	clockPolRisingButton->setChecked(1);

	QGroupBox		*	clockPhaseBox = new QGroupBox("Clock-Phase");
	QHBoxLayout		*	clockPhaseLayout = new QHBoxLayout;
	QRadioButton	*	clockPhaseRisingButton = new QRadioButton("Sample Daten bei der fhrenden Flanke");
	QRadioButton	*	clockPhaseFallingButton = new QRadioButton("Sample Daten bei der folgenden Flanke");
	m_clockPhaseConfigGroup = new QButtonGroup;

	clockPhaseLayout->addWidget(clockPhaseRisingButton);
	m_clockPhaseConfigGroup->addButton( clockPhaseRisingButton, 0 );
	clockPhaseLayout->addWidget(clockPhaseFallingButton);
	m_clockPhaseConfigGroup->addButton( clockPhaseFallingButton, 1 );
	clockPhaseBox->setLayout(clockPhaseLayout);
	clockPhaseRisingButton->setChecked(1);

#define		CPU_CLK		14745600

	QGroupBox		*	clockRateBox = new QGroupBox("Clock-Rate");
	QGridLayout		*	clockRateLayout = new QGridLayout;
	QRadioButton	*	clockRate1Button = new QRadioButton(tr("Prescaler 2 | ") + QString::number(CPU_CLK / 2 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate2Button = new QRadioButton(tr("Prescaler 4 | ") + QString::number(CPU_CLK / 4 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate3Button = new QRadioButton(tr("Prescaler 8 | ") + QString::number(CPU_CLK / 8 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate4Button = new QRadioButton(tr("Prescaler 16 | ") + QString::number(CPU_CLK / 16 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate5Button = new QRadioButton(tr("Prescaler 32 | ") + QString::number(CPU_CLK / 32 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate6Button = new QRadioButton(tr("Prescaler 64 | ") + QString::number(CPU_CLK / 64 / 1000.0f,'f',1) + tr(" kHz"));
	QRadioButton	*	clockRate7Button = new QRadioButton(tr("Prescaler 128 | ") + QString::number(CPU_CLK / 128 / 1000.0f,'f',1) + tr(" kHz"));

	m_clockRateConfigGroup = new QButtonGroup;

	clockRateLayout->addWidget(clockRate1Button,0,0);
	m_clockRateConfigGroup->addButton( clockRate1Button, 0 );
	clockRateLayout->addWidget(clockRate2Button,0,1);
	m_clockRateConfigGroup->addButton( clockRate2Button, 1 );
	clockRateLayout->addWidget(clockRate3Button,0,2);
	m_clockRateConfigGroup->addButton( clockRate3Button, 2 );
	clockRateLayout->addWidget(clockRate4Button,0,3);
	m_clockRateConfigGroup->addButton( clockRate4Button, 3 );
	clockRateLayout->addWidget(clockRate5Button,1,0);
	m_clockRateConfigGroup->addButton( clockRate5Button, 4 );
	clockRateLayout->addWidget(clockRate6Button,1,1);
	m_clockRateConfigGroup->addButton( clockRate6Button, 5 );
	clockRateLayout->addWidget(clockRate7Button,1,2);
	m_clockRateConfigGroup->addButton( clockRate7Button,6 );
	clockRateBox->setLayout(clockRateLayout);
	clockRate5Button->setChecked(1);


	QGroupBox	*	controlBox = new QGroupBox("SPI Steuerung");
	QGridLayout	*	controlLayout = new QGridLayout;
	QLabel	*	txLabel = new QLabel("TX-Daten");
	m_txDataEdit = new QLineEdit;
	QLabel	*	rxLabel = new QLabel("RX-Daten");
	m_rxDataEdit = new QLineEdit;
	m_sendDataButton = new QPushButton("senden");
	QRadioButton	*	dataModeBinButton = new QRadioButton("Binr");
	QRadioButton	*	dataModeHexButton = new QRadioButton("Hexadecimal");
	QRadioButton	*	dataModeDecButton = new QRadioButton("Decimal");
	m_setDataModeGroup = new QButtonGroup;
	QLabel	*	dataModeLabel = new QLabel("Anzeige-Modus:");

	m_rxDataEdit->setDisabled(1);

	m_setDataModeGroup->addButton( dataModeBinButton, 0);
	m_setDataModeGroup->addButton( dataModeHexButton, 1 );
	m_setDataModeGroup->addButton( dataModeDecButton, 2 );

	dataModeHexButton->setChecked(1);
	DataModeChangeButtonPressed(1);

	controlLayout->addWidget(txLabel,0,0);
	controlLayout->addWidget(rxLabel,1,0);
	controlLayout->addWidget(m_txDataEdit,0,1);
	controlLayout->addWidget(m_rxDataEdit,1,1);
	controlLayout->addWidget(dataModeLabel,0,2);
	controlLayout->addWidget(dataModeBinButton,0,3);
	controlLayout->addWidget(dataModeHexButton,1,3);
	controlLayout->addWidget(dataModeDecButton,2,3);
	controlLayout->setColumnStretch(4,10);
	controlLayout->addWidget(m_sendDataButton,2,5);


	controlBox->setLayout(controlLayout);

	QVBoxLayout	*	layout = new QVBoxLayout;
	layout->addWidget(dataOrderBox);
	layout->addWidget(clockPolBox);
	layout->addWidget(clockPhaseBox);
	layout->addWidget(clockRateBox);
	layout->addStretch(10);
	layout->addWidget(controlBox);
	setLayout(layout);
}

void	CSPIWidget::SetupSignals()
{
	connect(m_setDataModeGroup,SIGNAL(buttonClicked(int)),this,SLOT(DataModeChangeButtonPressed(int)));
	connect(m_sendDataButton,SIGNAL(clicked()),this,SLOT(SendDataButtonPressed()));
}

void	CSPIWidget::SpiDataReceived(quint8 data)
{
	QString s = m_rxDataEdit->text();

	if(s.length())
		s += QString(" ");
	s += QString::number(data,m_base);
	m_rxDataEdit->setText(s);
}

void	CSPIWidget::GetConfigFromGui(quint8 &spcr, quint8 &spsr)
{
	spcr = spsr = 0;

	switch( m_dataOrderConfigGroup->checkedId() )
	{
		case 0:
		{
			spcr |= (1<<5);
		}break;
		case 1:
		{
		}break;
		default:break;
	};
	switch(m_clockPolConfigGroup->checkedId() )
	{
		case 0:
		{
		}break;
		case 1:
		{
			spcr |= (1<<3);
		}break;
		default:break;
	};

	switch(m_clockPhaseConfigGroup->checkedId() )
	{
		case 0:
		{
		}break;
		case 1:
		{
			spcr |= (1<<2);
		}break;
		default:break;
	};

	switch(m_clockRateConfigGroup->checkedId() )
	{
		case 0: //2
		{
			spsr = 1;
		}break;
		case 1: //4
		{
		}break;
		case 2: //8
		{
			spcr |= 0x01;
			spsr = 1;
		}break;
		case 3: //16
		{
			spcr |= 0x01;
		}break;
		case 4: //32
		{
			spcr |= 0x02;
			spsr = 1;
		}break;
		case 5: //64
		{
			spcr |= 0x02;
		}break;
		case 6: //128
		{
			spcr |= 0x03;
		}break;
		default:break;
	};
}

void	CSPIWidget::SendDataButtonPressed()
{
	QString s = m_txDataEdit->text();

	if(s.isEmpty())
	{
		SHOW_INFO(tr("Keine Daten zum senden vorhanden"));
		return;
	}

	QStringList l = s.split(" ", QString::SkipEmptyParts);

	QString str;
	bool bOK;

	QList<quint8>	data;

	foreach(str, l)
	{
		int i = str.toInt(&bOK,m_base);
		if(bOK)
		{
			if (i < 0 || i > 0xFF)
			{
				bOK = 0;
				break;
			}
			data.append(i);
		}
	}

	if(bOK)
	{
		quint8 spcr, spsr;
		GetConfigFromGui(spcr, spsr);
		m_rxDataEdit->clear();
		emit SendSPIData(spcr, spsr, data);
	}
	else
		SHOW_ERROR("Ungltige Daten");
}

void	CSPIWidget::DataModeChangeButtonPressed(int id)
{
	m_txDataEdit->setText("");
	m_rxDataEdit->setText("");
	m_txDataEdit->setFocus(Qt::OtherFocusReason);

	switch(id)
	{
		case 0:
		{
			QRegExp	reg("[0-1]{1,8}(\\s[0-1]{1,8})*");
			m_dataValidator->setRegExp(reg);
			m_txDataEdit->setValidator(m_dataValidator);
			m_base = 2;
		}break;
		case 1:
		{
			QRegExp	reg("[0-9A-Fa-f]{1,2}(\\s[0-9A-Fa-f]{1,2})*");
			m_dataValidator->setRegExp(reg);
			m_txDataEdit->setValidator(m_dataValidator);
			m_base = 16;
		}break;
		case 2:
		{
			QRegExp	reg("(([0-1]?[0-9]?[0-9])|(2[0-4][0-9])|25[0-5])(\\s(([0-1]?[0-9]?[0-9])|(2[0-4][0-9])|25[0-5]))*");
			m_dataValidator->setRegExp(reg);
			m_txDataEdit->setValidator(m_dataValidator);
			m_base = 10;
		}break;
		default:
		{
			SHOW_ERROR( tr("Ungltige Auswahl - Decimal Ausgabe wird verwendet"));
			QRegExp	reg("(([0-1]?[0-9]?[0-9])|(2[0-4][0-9])|25[0-5])(\\s(([0-1]?[0-9]?[0-9])|(2[0-4][0-9])|25[0-5]))*");
			m_dataValidator->setRegExp(reg);
			m_txDataEdit->setValidator(m_dataValidator);
			m_base = 10;
		}break;
	}
}
