#include "ServoWidget.h"
#include "../ErrorHandling.h"

#define		DIAL_RANGE	1024
#define		POLL_SERVO_INTERVALL	20

CServoWidget::CServoWidget()
:CIOExpanderWidget(tr("Servos"),tr("Servos.png"))
{
	setupLayout();
	setupSignals();
}

void CServoWidget::ShowWidgetCallback()
{
	if(m_autoSetServosCheckbox->isChecked())
	{
		emitSendServoSignal();
		m_pollServoTimer->start(POLL_SERVO_INTERVALL);
	}
}

void CServoWidget::HideWidgetCallback()
{
	//disable the servos if another widget s selected
	m_pollServoTimer->stop();
	emit setOutput(0);
}

void CServoWidget::CloseWidgetCallback()
{
	m_pollServoTimer->stop();
	emit setOutput(0);
}

void CServoWidget::setupLayout()
{
	m_setServosButton = new QPushButton(tr("Servos bewegen"));
	m_autoSetServosCheckbox = new QCheckBox(tr("Servos automatisch bewegen"));
	m_pollServoTimer = new QTimer(this);

	m_minPulseBox = new QDoubleSpinBox;
	m_maxPulseBox = new QDoubleSpinBox;

	m_minPulseBox->setSuffix(tr(" ms"));
	m_maxPulseBox->setSuffix(tr(" ms"));

	m_minPulseBox->setRange(0.5,1.3);
	m_minPulseBox->setSingleStep(0.05);
	m_minPulseBox->setValue(1);

	m_maxPulseBox->setRange(1.7,2.5);
	m_maxPulseBox->setSingleStep(0.05);
	m_maxPulseBox->setValue(2);

	QLabel	*minLabel = new QLabel(tr("Minimale Impulslnge"));
	QLabel	*maxLabel = new QLabel(tr("Maximale Impulslnge"));

	QGroupBox	* servoBox = new QGroupBox(tr("Positionen"));
	QGridLayout * servoBoxesLayout = new QGridLayout;

	for( int n = 0; n < 8; n++)
	{
		QGroupBox	* box = new QGroupBox(tr("Servo %n","",n));
		QVBoxLayout *boxLayout = new QVBoxLayout;

		m_dials[n] = new QDial;
		m_dials[n]->setRange(0, DIAL_RANGE);
		m_dials[n]->setNotchesVisible(1);
		m_dials[n]->setNotchTarget(24);

		m_invCheckboxes[n] = new QCheckBox(tr("Richtung umkehren"));

		boxLayout->addWidget(m_dials[n]);
		boxLayout->addWidget(m_invCheckboxes[n]);
		box->setLayout( boxLayout );

		servoBoxesLayout->addWidget(box,n/4,n%4);
	}

	servoBox->setLayout(servoBoxesLayout);

	QGroupBox *ctrlBox = new QGroupBox(tr("Steuerung"));
	QVBoxLayout *ctrlLayout = new QVBoxLayout;
	ctrlBox->setLayout(ctrlLayout);

	ctrlLayout->addWidget( m_setServosButton );
	ctrlLayout->addWidget( m_autoSetServosCheckbox );

	QGroupBox *pulseBox = new QGroupBox(tr("Impulslngen"));
	QGridLayout *pulseLayout = new QGridLayout;
	pulseBox->setLayout( pulseLayout );

	pulseLayout->addWidget( minLabel,0,0 );
	pulseLayout->addWidget( maxLabel,1,0 );
	pulseLayout->addWidget( m_minPulseBox,0,1 );
	pulseLayout->addWidget( m_maxPulseBox,1,1 );

	QVBoxLayout *layout = new QVBoxLayout;
	QHBoxLayout *dummyLayout = new QHBoxLayout;

	dummyLayout->addWidget( ctrlBox );
	dummyLayout->addStretch(10);
	dummyLayout->addWidget( pulseBox );

	layout->addWidget(servoBox);
	layout->addStretch(10);
	layout->addLayout(dummyLayout);


	setLayout( layout );
}

void CServoWidget::setupSignals()
{
	for(int i = 0; i < 8; i++)
	{
		connect(m_invCheckboxes[i], SIGNAL(toggled(bool)),this,SLOT(invCheckBoxToggled(bool)));
	}

	connect(m_pollServoTimer, SIGNAL(timeout()),this,SLOT( emitSendServoSignal() ));

	connect(m_setServosButton, SIGNAL(clicked()),this,SLOT(emitSendServoSignal()));
	connect(m_autoSetServosCheckbox, SIGNAL(toggled(bool)),this,SLOT(autoSetCheckBoxToggled(bool)));
}

QList<float> CServoWidget::getServoValuesFromGui()
{
	QList<float> values;
	for(int i = 0; i < 8; i++)
	{
		float dial_value = static_cast<float>(m_dials[i]->value()) / static_cast<float>(DIAL_RANGE);

		if(m_invCheckboxes[i]->isChecked())
			values << 1.0 - dial_value;
		else
			values << dial_value;
	}

	return values;
}

void CServoWidget::autoSetCheckBoxToggled(bool b)
{
	if(b)
	{
		m_pollServoTimer->start(POLL_SERVO_INTERVALL);
		emitSendServoSignal();
	}
	else
		m_pollServoTimer->stop();
}

void CServoWidget::invCheckBoxToggled(bool)
{
	if(m_autoSetServosCheckbox->isChecked())
		emitSendServoSignal();
}

void CServoWidget::emitSendServoSignal()
{
	emit setServos( getServoValuesFromGui() , m_minPulseBox->value(), m_maxPulseBox->value() );
}
