#pragma once

#include "USBCommunicationModule.h"

#include <QBitArray>

#define		CMD_A2D				'a'	//sample analog data
#define		CMD_FREQ			'f' //set frequency
#define		CMD_OUT				'o'	//set digital output
#define		CMD_IN				'i'	//read digital input
#define		CMD_PATTERN			'p'	//set pattern
#define		CMD_SAMPLE			's'	//sample digital data
#define		CMD_LOGIC_OP	 	'x'	//set pattern and sample
#define		CMD_RELAIS		 	'r'	//set the relais state
#define		CMD_SPI_TRANSFER 	't'	//transfer spi data
#define		CMD_SPI_CONFIG		'c'	//config the spi bus
#define		CMD_SET_TRIGGER		'z'	//set a trigger for the next pattern/sample operation
#define		CMD_RESET			'y'
#define		CMD_SET_SERVO		'm'
#define		CMD_SOFTWARE_VER	'v'



class CIOExpanderCommunicationModule
:public CUSBCommunicationModule
{
	Q_OBJECT
public:
	CIOExpanderCommunicationModule();

private:
	void SetFrequency(quint32);
	void SetTrigger(quint16 trigger1, quint16 trigger2);
	void ConfigSPI(quint8 spcr, quint8 spsr);
	bool CheckConnetion();
	void ProcessReceivedData(CUSBCommand usbCmd);
	void ConnectionEstablishedCallback();

public slots:
	void RequestSW();
	void Reset();

	void RequestSamples(quint8 num, quint16 frequ, quint16 trigger1, quint16 trigger2);
	void SendPattern(QBitArray *pattern, quint16 frequ, quint16 trigger1, quint16 trigger2);
	void RequestSamplesAndSendPattern(QBitArray *pattern, quint16 frequ, quint16 trigger1, quint16 trigger2);

	void SetOutput(quint16 out);
	void SetRelais(quint8 out);
	void PollInput();
	void PollADC(quint8 channels);

	void SendSPIData(quint8 spcr, quint8 spsr, QList<quint8> data);
	void SetServos(QList<float> values,float min,float max);

signals:
	void 	NewSamplesReceived(QBitArray *pattern);
	void	NewInputReceived(quint16 state);
	void	NewADCValueReceived(quint8 value, quint8 channel);
	void	NewSPIDataReceived(quint8 data);
	void	NewSWVersion(QString ver);

private:

	struct
	{
		quint32		frequency;
		quint8		spsr;
		quint8		spcr;
		quint16		trigger1;
		quint16		trigger2;
	}m_configData;

	static unsigned int		s_cpuFrequency;
};
