#include "USBCommunicationModule.h"

#include <QEventLoop>
#include <QTextStream>


//write transfered/received data to stdout by defining this macros
//#define		DBG_TX_DATA
//#define		DBG_RX_DATA


#define		TX_BUFFER_SIZE		25
#define		RX_BUFFER_SIZE		35

#if defined(DBG_TX_DATA) || defined(DBG_RX_DATA)
static QTextStream cout(stdout);
#endif


CUSBCommunicationModule::CUSBCommunicationModule(const QString &device, int pollIntervall)
{
	m_deviceName = device;
	m_open = 0;
	m_pollIntervall = pollIntervall;
}

void CUSBCommunicationModule::run()
{
	QEventLoop loop; //create an event loop, needed for the timer
	OpenUSBDevice();

	if(!m_open)
		emit ConnectionLost();

	QTimer *pollTimer = new QTimer(this);
	connect(pollTimer, SIGNAL(timeout()), this, SLOT(PollUSB()));
	pollTimer->start( m_pollIntervall );

	loop.exec();

	CloseUSBDevice();

	delete pollTimer;
}

void CUSBCommunicationModule::OpenUSBDevice()
{
	if(m_open)
		return;

	FT_STATUS ftStatus;

	char * buffer;
	buffer = new char[m_deviceName.length()+1];
	for(int i = 0; i < m_deviceName.length();i++)
	{
		buffer[i] = m_deviceName[i].toAscii();
	}
	buffer[m_deviceName.length()] = 0;

	ftStatus = FT_OpenEx(buffer,FT_OPEN_BY_DESCRIPTION,&m_handle);

	delete[] buffer;

	if(ftStatus != FT_OK)
		m_open = 0;
	else
	{
		FT_SetBaudRate(m_handle, FT_BAUD_19200);
		FT_SetDataCharacteristics( m_handle, FT_BITS_8, FT_STOP_BITS_1, FT_PARITY_NONE);

		m_open = 1;
		emit ConnectionEstablished();
		ConnectionEstablishedCallback();
	}
}

void CUSBCommunicationModule::CloseUSBDevice()
{
	if(m_open)
		FT_Close(m_handle);
}

#define min(a,b) a<b?a:b

void CUSBCommunicationModule::PollUSB()
{
	if(!m_open)
	{
		OpenUSBDevice();
		return;
	}

	FT_STATUS ftStatus;
	DWORD EventDWord;
	DWORD TxBytes;
	DWORD RxBytes;
	DWORD BytesReceived;
	quint8 RxBuffer[RX_BUFFER_SIZE];
	unsigned int BytesToWrite;
	DWORD 	BytesWritten;
	quint8 	TxBuffer[TX_BUFFER_SIZE];

	FT_GetStatus(m_handle,&RxBytes,&TxBytes,&EventDWord);

	if (RxBytes > 0) //there is data to read
	{
		RxBytes = min(RxBytes,RX_BUFFER_SIZE);

		ftStatus = FT_Read(m_handle,RxBuffer,RxBytes,&BytesReceived);
		if (ftStatus != FT_OK)//error
		{
			m_open = 0;
			emit ConnectionLost();
			return;
		}

		if(m_commands.size())
		{
			unsigned int bytesToCopy = min(BytesReceived,m_commands.first().m_answerBytes);
#ifdef DBG_RX_DATA
			cout << "rxdata" << endl;
#endif
			for(unsigned int n = 0; n < bytesToCopy; n++) //copy data
			{
#ifdef DBG_RX_DATA
				cout << RxBuffer[n] << " " ;
#endif
				m_commands.first().m_answer << RxBuffer[n];
				m_commands.first().m_answerBytes--;
			}
#ifdef DBG_RX_DATA
			cout << endl;
#endif
			if(m_commands.first().m_answerBytes == 0)
			{
				ProcessReceivedData(m_commands.first());
				m_commands.removeFirst();
			}
		}
	}

	if(m_commands.isEmpty() || m_commands.first().m_data.isEmpty())
		return;

	if( m_commands.first().m_data.size() < TX_BUFFER_SIZE)
		BytesToWrite = m_commands.first().m_data.size();
	else
		BytesToWrite = TX_BUFFER_SIZE;

#ifdef DBG_TX_DATA
	cout << "txdata" << endl;
#endif
	for(unsigned int n = 0; n < BytesToWrite;n++)
	{
		TxBuffer[n] = m_commands.first().m_data.takeFirst();
#ifdef DBG_TX_DATA
		cout << (char)TxBuffer[n] << "  ";
#endif
	}
#ifdef DBG_TX_DATA
	cout << endl;
#endif

	if(m_commands.first().m_data.isEmpty() && m_commands.first().m_answerBytes == 0)
		m_commands.takeFirst();

	ftStatus = FT_Write(m_handle, &TxBuffer, BytesToWrite, &BytesWritten );

	if (ftStatus != FT_OK)
	{
		emit ConnectionLost();
		m_open = 0;
		return;
	}
}

bool	CUSBCommunicationModule::IsConnected()
{
	return m_open;
}

void 	CUSBCommunicationModule::SendCommand(CUSBCommand cmd)
{
	m_commands << cmd;
}
